{-------------------------------------------------------------------------------
  :: Iczelion's PE Tutorial 5 for Delphi ::
  translated by Sunshine

  Shows the Section Table of a PE File in a listview using Filemapping.

  Have a look at readme.txt for further information.

  Sunshine, February 2003
-------------------------------------------------------------------------------}

program Petutno5;

uses
  Windows, Messages, CommCtrl, Commdlg, PElib;

const
  WindowWidth   = 490;
  WindowHeight  = 200;

  IDC_LV = 100;       // constant for our Listview

var
  hLV     : Cardinal; // Handle of Listview

  //OPENFILENAME
  ofn : OPENFILENAME;
  filterstring : LPCTSTR =
                 'Executable Files (*.exe, *.dll)'+#0+'*.exe;*.dll'+#0+
                 'All Files (*.*)'+#0+'*.*'+#0+#0;
  filename : array[0..MAX_PATH] of char;


  MyFile   : THandle;
  hmapping : THandle;
  pmapping : Pointer;

  Pdosh         : PIMAGE_DOS_HEADER;
  peh           : PIMAGE_NT_HEADERS;
  sectionh      : PIMAGE_SECTION_HEADER;
  section_count : Integer;

{$R icon.RES}

procedure GetSectioninfos(Filename : pchar);
var
  i : Integer;
  lvi   : TLVItem;
  buffer: array [0..19] of Char;
begin
  MyFile := CreateFileA(filename, GENERIC_READ, FILE_SHARE_READ, nil,
            OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);

  hmapping := CreateFileMapping(MyFile, nil, PAGE_READONLY, 0, 0, nil);
  pmapping := MapViewOfFile(hMapping, FILE_MAP_READ, 0, 0, 0);

  pdosh :=  PIMAGE_DOS_HEADER(pmapping);
  peh:= PIMAGE_NT_HEADERS(Longword(pdosh) + Longword(pdosh.e_lfanew));
  if (pdosh.e_magic = IMAGE_DOS_SIGNATURE) and (peh.Signature = IMAGE_NT_SIGNATURE)
  then           // If valid PE File go on
  begin
    section_count := peh.FileHeader.NumberOfSections;
    //Sectiontable
    sectionh := PIMAGE_SECTION_HEADER(LongWord(peh) + Longword(sizeof(IMAGE_NT_HEADERS)));


    for i := 1 to section_count do   // loop through each section
    begin
      lvi.mask     := LVIF_TEXT;
      lvi.iItem :=    i-1;           // row
      lvi.iSubItem := 0;             // column
      lvi.pszText := sectionh.Name;
      SendMessage(hLV, LVM_INSERTITEM, 0, Integer(@lvi));

      lvi.iSubItem := 1;
      lvi.mask := LVIF_TEXT;
      wvsprintf(buffer, '%08X', PChar(@sectionh.PhysicalAddress));
      lvi.pszText := buffer;
      SendMessage(hLV, LVM_SETITEM, 0, Integer(@lvi));

      lvi.iSubItem := 2;
      lvi.mask := LVIF_TEXT;
      wvsprintf(buffer, '%08X', PChar(@sectionh.VirtualAddress));
      lvi.pszText := buffer;
      SendMessage(hLV, LVM_SETITEM, 0, Integer(@lvi));

      lvi.iSubItem := 3;
      lvi.mask := LVIF_TEXT;
      wvsprintf(buffer, '%08X', PChar(@sectionh.SizeofRawdata));
      lvi.pszText := buffer;
      SendMessage(hLV, LVM_SETITEM, 0, Integer(@lvi));

      lvi.iSubItem := 4;
      lvi.mask := LVIF_TEXT;
      wvsprintf(buffer, '%08X', PChar(@sectionh.PointertoRawData));
      lvi.pszText := buffer;
      SendMessage(hLV, LVM_SETITEM, 0, Integer(@lvi));

      lvi.iSubItem := 5;
      lvi.mask := LVIF_TEXT;
      wvsprintf(buffer, '%08X', PChar(@sectionh.Characteristics));
      lvi.pszText := buffer;
      SendMessage(hLV, LVM_SETITEM, 0, Integer(@lvi));

      sectionh := PIMAGE_SECTION_HEADER(Longword(sectionh) + longword(sizeof(IMAGE_SECTION_HEADER)));
    end;

  end
  else
    Messagebox(0, pchar('Not a real PE File!'), pchar('Error'), MB_OK);

  UnmapViewofFile(pmapping);
  CloseHandle(hmapping);
  CloseHandle(MyFile);
end;
  


function WndProc(hWnd: HWND; uMsg: UINT; wParam: wParam; lParam: LParam): LRESULT; stdcall;
var
  x, y : integer;
  lvc : TLVColumn;
begin
  Result := 0;
  case uMsg of
    WM_CREATE:
    begin
      // Center window
      x := GetSystemMetrics(SM_CXSCREEN);
      y := GetSystemMetrics(SM_CYSCREEN);
      MoveWindow(hWnd, (x div 2) - (WindowWidth div 2),
        (y div 2) - (WindowHeight div 2),
        WindowWidth, WindowHeight, true);

      // Create Listview
      hLV := CreateWindowEx(WS_EX_CLIENTEDGE, 'SysListView32', '', WS_VISIBLE or
        WS_CHILD or LVS_REPORT, 0, 0, 100, 100, hWnd, IDC_LV, hInstance, nil);

      // Set icon
      SendMessage( hwnd, WM_SETICON, ICON_BIG,
                               LoadIcon(hinstance,PCHAR('MAINICON')));

      // Create Columns
      lvc.mask := LVCF_TEXT or LVCF_WIDTH or LVCF_FMT;
      lvc.pszText := 'Section';
      lvc.cx := 80;
      SendMessage(hLV, LVM_INSERTCOLUMN, 0, Integer(@lvc));

      lvc.fmt := LVCFMT_LEFT;
      lvc.pszText := 'Virtual Size';
      lvc.cx := 80;
      SendMessage(hLV, LVM_INSERTCOLUMN, 1, Integer(@lvc));

      lvc.fmt := LVCFMT_LEFT;
      lvc.pszText := 'Virtual Address';
      lvc.cx := 80;
      SendMessage(hLV, LVM_INSERTCOLUMN, 2, Integer(@lvc));

      lvc.fmt := LVCFMT_LEFT;
      lvc.pszText := 'Raw Size';
      lvc.cx := 80;
      SendMessage(hLV, LVM_INSERTCOLUMN, 3, Integer(@lvc));

      lvc.fmt := LVCFMT_LEFT;
      lvc.pszText := 'Raw Offset';
      lvc.cx := 80;
      SendMessage(hLV, LVM_INSERTCOLUMN, 4, Integer(@lvc));

      lvc.fmt := LVCFMT_LEFT;
      lvc.pszText := 'Charac.';
      lvc.cx := 80;
      SendMessage(hLV, LVM_INSERTCOLUMN, 5, Integer(@lvc));

    end;


    WM_DESTROY:  PostQuitMessage(0);

    WM_SIZE:
    begin
      MoveWindow(hLV, 0, 0, LoWord(lParam), HiWord(lParam), true)
    end;
  else
    Result := DefWindowProc(hWnd, uMsg, wParam, lParam);
  end;
end;



var
  wc: TWndClassEx = (
    cbSize          : SizeOf(TWndClassEx);
    Style           : CS_HREDRAW or CS_VREDRAW;
    lpfnWndProc     : @WndProc;
    cbClsExtra      : 0;
    cbWndExtra      : 0;
    hbrBackground   : COLOR_APPWORKSPACE;
    lpszMenuName    : MAKEINTRESOURCE(100);
    lpszClassName   : 'PETutNo5';
    hIconSm         : 0;
  );
  msg: TMsg;

begin

  ofn.lStructSize := sizeof(ofn);
  ofn.hInstance   := HInstance;
  ofn.lpstrFilter := filterstring;
  ofn.lpstrFile := filename;
  ofn.nMaxFile :=  512;
  ofn.lpstrTitle := 'Choose file...';
  ofn.Flags := OFN_FILEMUSTEXIST or OFN_PATHMUSTEXIST or OFN_LONGNAMES or
              OFN_EXPLORER or OFN_HIDEREADONLY;
  if Getopenfilename(ofn) then
  begin
    // make sure that the common control dynamic-link library (DLL) is loaded.
    InitCommonControls;
    
    wc.hInstance  := hInstance;
    wc.hIcon      := LoadIcon(0, IDI_APPLICATION);
    wc.hCursor    := LoadCursor(0, IDC_ARROW);
    RegisterClassEx(wc);
    CreateWindowEx(0, 'PETutNo5', 'Iczelion' +#39 +'s PE Tutorial 5 for Delphi',
                   WS_CAPTION or WS_VISIBLE or WS_SYSMENU,
                   Integer(CW_USEDEFAULT),Integer(CW_USEDEFAULT),
                   WindowWidth, WindowHeight, 0, 0, hInstance, nil);

    GetSectioninfos(filename);
  end
  else
  ExitProcess(0);

             
  while true do
  begin
    if not GetMessage(msg, 0, 0, 0) then
      break;
    begin
      TranslateMessage(msg);
      DispatchMessage(msg);
    end;
  end;
  ExitCode := msg.wParam;
end.
